"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PriorityQueue = void 0;
const Bucket_1 = require("./Bucket");
/**
 * Priority Queue
 *
 * A bucket queue supporting a dynamic number of buckets
 * ```typescript
 * const queue = new PriorityQueue(3);
 * // queue = [[],[],[]]
 * queue.enqueue(() => {}, 1);
 * // queue = [[() => {}],[],[]]
 * queue.enqueue(() => {}, 2);
 * // queue = [[() => {}],[],[() => {}]]
 * queue.dequeue();
 * // queue = [[],[],[() => {}]]
 * queue.dequeue();
 * // queue = [[],[],[]]
 * ```
 */
class PriorityQueue {
    constructor(buckets) {
        this.buckets = [];
        this.max = buckets - 1;
        for (let i = 0; i < buckets; i++) {
            this.buckets.push(new Bucket_1.Bucket());
        }
    }
    /**
     * Enqueue
     *
     * Adds an item to the queue at the specified priority.
     * If no priority is provided, the item will be indexed
     * into the *highest* priority bucket
     */
    enqueue(item, priority = 0) {
        this.guard(priority);
        return this.buckets[priority].enqueue(item);
    }
    /**
     * Dequeue
     *
     * Removes the first item in the priority queue and
     * returns it. Items are dequeued from higher priorities
     * before lower priorities
     */
    dequeue() {
        return this.findFirst((bucket) => {
            return bucket.dequeue();
        });
    }
    /**
     * Get Bucket
     *
     * Returns the Nth bucket in the Queue
     */
    getBucket(priority = 0) {
        this.guard(priority);
        return this.buckets[priority];
    }
    /**
     * Peak
     *
     * Returns the highest priority item in the Queue
     */
    peak() {
        return this.findFirst((bucket) => {
            return bucket.peak();
        });
    }
    /**
     * IsEmpty
     *
     * Returns true if the Queue has no items in any of
     * its buckets
     */
    get isEmpty() {
        return this.findFirst(() => {
            return false;
        }) === undefined
            ? true
            : false;
    }
    /**
     * Length
     *
     * Returns the total number of items in the Queue
     */
    get length() {
        let length = 0;
        this.all((bucket) => {
            length += bucket.length;
        });
        return length;
    }
    /**
     * Delete By ID
     *
     * Removes an item from the Queue using its ID and
     * priority
     */
    delete(ID, priority) {
        this.guard(priority);
        return this.getBucket(priority).delete(ID);
    }
    /**
     * Clear
     *
     * Resets the priority queue and removes all items
     */
    clear() {
        this.all((bucket) => {
            bucket.clear();
        });
    }
    /**
     * Guard
     *
     * Throws a range error if the priority specified is out of range
     */
    guard(priority) {
        if (priority > this.max) {
            throw new Error(`Out of Range Error: Attempted to access a bucket index that does not exist`);
        }
    }
    /**
     * Find First
     *
     * Executes the provided callback on the first non-empty
     * bucket
     */
    findFirst(cb) {
        for (const bucket of this.buckets) {
            if (!bucket.isEmpty) {
                return cb(bucket);
            }
        }
    }
    /**
     * All
     *
     * Executes the provided callback on each of the buckets
     * present in the Queue
     */
    all(cb) {
        for (const bucket of this.buckets) {
            cb(bucket);
        }
    }
}
exports.PriorityQueue = PriorityQueue;
